﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NFlickr.Communication;
using NFlickr.Factory;
using NFlickr.Core.Interface;

namespace NFlickr.Core
{
    public class Token : RequireSignedServiceBase
    {

        internal Token()
        {

        }

        public override string MethodName
        {
            get { return "flickr.auth.getToken"; }
        }

        public override string TagName
        {
            get { return "auth/token"; }
        }

        public Frob Frob
        {
            get;
            set;
        }

        private string _value;
        public string Value
        {
            get
            {
                if (string.IsNullOrEmpty(_value))
                {
                    if (Refresh())
                    {
                        return _value;
                    }
                    else
                    {
                        return null;
                    }
                }
                return _value;
            }
        }

        public User User
        {
            get;
            set;
        }

        public override bool Refresh()
        {
            IChannel channel = FlickrChannelFactory.CreateInstance();

            channel.Complete +=new EventHandler<SendRequestCompletedEventArgs>(channel_Complete);
            channel.SignatureRequired += new NFlickr.Core.Interface.SignatureRequiredEventHandler(channel_SignatureRequired);


            Params = new Dictionary<string, string>();

            Params.Add(NFlickr.Resource.FlickrService.ApplicationKeyName, ApplicationKey);
            Params.Add(NFlickr.Resource.FlickrService.MethodKeyName, MethodName);
            Params.Add(NFlickr.Resource.FlickrService.FrobKeyName, Frob.Value);
            IResult result = channel.SendRequest(FlickrServiceUrl, this);

            if (result != null && result.ResultCode == 0)
            {
                _value = result.ResultString;
                return true;
            }
            return false;
        }

        string channel_SignatureRequired(object sender, NFlickr.Core.Interface.SignatureRequiredEventArgs args)
        {
            Params.Add(NFlickr.Resource.FlickrService.SignatureKeyName, args.Singnature);
            return GetQueryString(Params);
        }

        void channel_Complete(object sender, NFlickr.Core.Interface.SendRequestCompletedEventArgs args)
        {

            args.Result = ParseResult(args.Response);
            if (args.Result.ResultCode == 0)
            {
                User user = null;
                if (User.TryParse(args.Response, out user))
                {
                    User = user;
                }
            }
        }
    }
}
